# GNU GPL

import inspect
import os
import string
import re
import pickle
import subprocess
import ConfigParser

from xml.sax.handler import ContentHandler
from xml.sax import make_parser
from xml.sax.saxutils import escape

# Gamelist interface. Contains all functions that works with gamelist.
# This class don't need pygtk.
class GameList:
    def __init__(self,config):
        self.config=config
        self.list=dict()
        self.match=dict()
        self.error=self.print_error
        self.progress=self.print_progress
        
    def write_xml(self,file):
        fd=open(file,'w')
        fd.write('<?xml version="1.0" ?>\n')
        fd.write('<games>\n')
        for game in self.list:
            fd.write('<game name="%s">' % game)
            for name in self.config["gamelist_fields"]:
                fd.write('<%s>%s</%s>' % (name,escape(self.list[game][name].encode('utf-8')),name))
            fd.write('</game>\n')
        fd.write('</games>\n')
        return

    def build_list(self):
        if not os.access(os.path.dirname(self.config["gamelist"]),os.W_OK):
            self.config["gamelist"]=os.environ['HOME']+"/.pyxmame/gamelist.xml"
            self.config["gamelist_format"]="xml"
            if not os.path.isdir(os.path.dirname(self.config["gamelist"])):
                os.makedirs(os.path.dirname(self.config["gamelist"]))
            
        if os.access(self.config["catver"],os.R_OK):
            self.load_ini_file(self.config["catver"])
        
        p=subprocess.Popen(["which",self.config["binary"]],close_fds=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        if len(p.stdout.readline()) == 0:
            self.error(_("Binary '%s' not found" % self.config["binary"]))
            return
        p=subprocess.Popen([self.config["binary"],self.config["xml_param"]],close_fds=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        self.load_xml(p.stdout)
        
        self.verify_roms()
        
        self.progress(bar=1)
        if self.config["gamelist_format"] == "xml":
            self.write_xml(self.config["gamelist"])
        elif self.config["gamelist_format"] == "python":
            pickle.dump(self.list,open(self.config["gamelist"],"w"))
        else:
            self.error(_("Unknown gamelist format: %s" % self.config["gamelist_format"]))


    def load_ini_file(self,file):
        confparser= ConfigParser.RawConfigParser()
        confparser.read(file)
        for item in self.config["gamelist_fields"]:
            title=item.title()
            if confparser.has_section(title):
                for game,value in confparser.items(title):
                    if not self.list.has_key(game):
                        self.list[game]=dict()
                        for name in self.config["gamelist_fields"]:
                            self.list[game][name]=""
                        self.list[game]["state"]="unknown"
                        self.list[game]["rom"]=game
                    self.list[game][item]=value

    def load_xml(self,source):
        handler = GameHandler(self.config,self.list,self.progress)
        saxparser = make_parser()
        saxparser.setContentHandler(handler)
        saxparser.parse(source)
        self.list=handler.games

    def load_list(self):
        if not os.access(self.config["gamelist"],os.R_OK):
             self.error(_("File %s not found" % self.config["gamelist"]))
             return
        self.progress(bar=0.01)
        if self.config["gamelist_format"] == "xml":
            self.load_xml(open(self.config["gamelist"]))
        elif self.config["gamelist_format"] == "python":
            self.list = pickle.load(open(self.config["gamelist"]))
        else:
            self.error(_("Unknown format: %s" % self.config["gamelist_format"]))
        self.progress(bar=1)
        self.match=self.list

    def verify_roms(self):
        argv=list()
        argv.append(self.config["binary"])
        argv.append(self.config["rompath_param"])
        argv.append(self.config["rompath"])
        argv.append(self.config["verify_param"])
        p=subprocess.Popen(argv,close_fds=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        
        for line in p.stdout:
            game = line[0:10].strip()
            if self.list.has_key(game):
                self.list[game]["state"]=line[10:-1].strip()
            self.progress()
            self.progress()

    # returns a dictionary of lists to select filter
    def get_selects_lists(self):
        lists=dict()
        for name in self.config["selects"]:
            lists[name]=list()
        for game in self.match:
            if self.match[game]["state"] not in self.config["hide_rows"]:        
                for name in self.config["selects"]:
                    if self.match[game][name] not in lists[name]:
                        lists[name].append(self.match[game][name])
        for name in self.config["selects"]:
            lists[name].sort()
            lists[name].insert(0,_("Any"))
        return lists

    # returns the games that match the given pattern
    def match_pattern(self,match_list,name,pattern):
        rx=re.compile(pattern,re.I)
        match=dict()
        for game in match_list:
            if rx.search(match_list[game][name]):
                match[game]=match_list[game]
        return match

    # returns the games that match all patterns
    def get_list(self,pattern):
        self.match=None
        self.match=self.list
        for name in pattern:
            if pattern[name] != "":
                self.match=self.match_pattern(self.match,name,pattern[name])
        return self.match
    
    # returns the command to execute a game
    def get_command(self,game):
        argv=list()
        argv.append(self.config["binary"])
        for extra in self.config["extra_param"].split(" "):
            argv.append(extra)
        argv.append(self.config["rompath_param"])
        argv.append(self.config["rompath"])
        argv.append(game)
        return argv
        
    # default functions
    def print_error(self,text):
        print text
        return
    def print_progress(self,bar):
        return

    # sets error function
    def set_error(self,error):
        self.error=error
    # sets progress function
    def set_progress(self,progress):
        self.progress=progress

# Handler to load the xmame output or gamelist
class GameHandler(ContentHandler):
    def __init__(self,config,gamelist,progress):
        ContentHandler.__init__(self)
        self.record = False
        self.games=gamelist
        self.progress=progress
        self.names=list(config["gamelist_fields"])
        if "state" not in config["gamelist_fields"]:
            self.names.append("state")

    def startElement(self, name, attrs):
        if name == "game":
            self.name = attrs.get("name");
            if not self.games.has_key(self.name):
                self.games[self.name]=dict()
                for item in self.names:
                    self.games[self.name][item]=""
                self.games[self.name]["state"]="unknown"
        elif name in self.names:
            self.buffer=""
            self.record=True

    def endElement(self, name):
        if name in self.names:
            self.games[self.name][name] = self.buffer
            self.record=False
        elif name == "game":
            self.games[self.name]["rom"]=self.name
            self.progress()

    def characters(self, data):
        if self.record:
            self.buffer += data
