#!/usr/bin/env python
# GNU GPL

import string
import pango
import GameList
import Config
import Games
import Selects
import Show
import Filter
import Preferences
import View
import Progress
import gettext
import os
import subprocess

import pygtk
pygtk.require('2.0')
import gtk
import gobject

# Main class
class Main(gobject.GObject):
    def __init__(self):
    
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_default_size(800,700)
        self.window.connect("delete_event", self.exit)
        self.window.connect("key-press-event",self.key_press_event)
        self.window.set_border_width(10)
        self.window.set_title("PyXmame")
        
        gettext.install('pyxmame','../locale',codeset='ISO-8859-15')

        # load configuration
        self.config=Config.Config()
        self.config.read()
        self.config.set_error(self.error)

        # select filter box and combobox box
        self.selects_box = Selects.Selects(
                                    self.config,
                                    self.filter_selection)

        # games list box
        self.list_box = Games.Games(
                                    self.config,
                                    self.start_xmame,
                                    self.show_game)

        # show games box
        self.game_box=Show.Show(self.config)

        # log game execution box
        self.view_box=View.View()

        # down filter button
        butt = gtk.Button(stock=gtk.STOCK_GO_FORWARD)
        butt.connect("clicked", self.filter_selection, None)
        alignment = butt.get_children()[0]
        hbox = alignment.get_children()[0]
        image, label = hbox.get_children()
        label.set_text(_('Filter'))

        evtbox=gtk.EventBox()
        evtbox.add(butt)
        self.tooltips=gtk.Tooltips()
        self.tooltips.set_tip(evtbox, _("Filter games"))

        left_box = gtk.VBox(False, 10)
        left_box.pack_start(self.selects_box, True, True, 0)
        left_box.pack_start(evtbox, False, True, 0)

        up_box = gtk.HBox(False, 10)
        up_box.pack_start(left_box, False, True, 0)
        up_box.pack_start(self.list_box, True, True, 0)
        up_box.pack_start(self.view_box, True, True, 0)

        # create user interface (toolbar and some direct key)
        self.ui = self.create_ui()

        vbox = gtk.VBox(False,0)
        vbox.pack_start(self.ui.get_widget("/Toolbar"),False)
        vbox.pack_start(up_box, True, True, 0)
        vbox.pack_start(self.game_box, False, True, 0)
        self.window.add(vbox)

        self.window.show_all()
        self.view_box.hide_all()
        self.window.maximize()

        # create gamelist interface
        self.gamelist=GameList.GameList(self.config)
        # set error function
        self.gamelist.set_error(self.error)
        
        # load gamelist
        if os.access(self.config["gamelist"],os.R_OK):
            self.load()
        
        # show start message because the gamlist don't exist
        if not os.access(self.config["gamelist"],os.R_OK):
            self.start_message()

    # create user interface
    def create_ui(self):
        ui_string="""
        <ui>
            <toolbar name="Toolbar">
              <toolitem action="menu-execute"/>
              <toolitem action="menu-find"/>
              <toolitem action="menu-filter"/>
              <separator />
              <toolitem action="menu-preferences"/>
              <toolitem action="menu-about"/>
              <toolitem action="menu-exit"/>
            </toolbar>
            <accelerator action="menu-find"/>
            <accelerator action="menu-exit"/>
        </ui>
        """
        ui = gtk.UIManager()
        ui.add_ui_from_string(ui_string)
        
        actions = [("menu-find",gtk.STOCK_FIND, None, None, _("Find games"), self.filter_dialog),
                   ("menu-filter",gtk.STOCK_GO_FORWARD, None, None, _("Filter games"), self.filter_selection),
                   ("menu-execute",gtk.STOCK_EXECUTE, None, None, _("Execute the selected game"), self.start_xmame),
                   ("menu-preferences",gtk.STOCK_PREFERENCES, None, None, _("Change preferences"), self.preferences_dialog),
                   ("menu-about",gtk.STOCK_ABOUT, None, None, _("About"), self.about),
                   ("menu-exit",gtk.STOCK_QUIT, None, None, _("Exit program"), self.exit),]

        ag = gtk.ActionGroup("Menu")
        ag.add_actions(actions)
        
        ui.insert_action_group(ag, 0)
        ui.get_widget("/Toolbar").set_style(gtk.TOOLBAR_ICONS)

        accelgroup = ui.get_accel_group()
        self.window.add_accel_group(accelgroup)

        return ui

    # execute xmame binary
    def start_xmame(self,treeview,path=None,view_column=None):
        selection=self.list_box.get_selection()
        if selection==None:
            return
        
        self.list_box.hide()
        self.view_box.show_all()
        
        command=self.gamelist.get_command(selection)
        p=subprocess.Popen(command,close_fds=True,stderr=subprocess.STDOUT,stdout=subprocess.PIPE)
        
        line=string.join(command," ")+"\n"
        while line != "":
            self.view_box.insert(line)
            while gtk.events_pending():
                gtk.main_iteration_do(False)
            line=p.stdout.readline()

        self.view_box.hide()
        self.list_box.show_all()
        self.view_box.initialize()
        self.list_box.set_focus()
        return

    # when game selected changes, the down box show the game information
    def show_game(self,treeview):
        game=self.list_box.get_selection()
        if game == None:
            return
        self.game_box.set_game(self.gamelist.list[game])
        return

    def exit(self, widget, event=None, data=None):
        gtk.main_quit()
        return True
       
    # open find dialog and set games list shown with find values
    def filter_dialog(self,widget=None,event=None):
        dlg = Filter.Filter(self.config,self.window)
        if dlg.run() != gtk.RESPONSE_ACCEPT:
            dlg.destroy()
            return
        patterns=dlg.get_values()
        dlg.destroy()
        match=self.gamelist.get_list(patterns)
        self.list_box.set_list(match)
        self.selects_box.initialize()
        return

    # open preferences dialog
    def preferences_dialog(self,widget=None,event=None):
        dlg = Preferences.Preferences(self.error,self.config)
        response= dlg.run()
        dlg.destroy()
        # load images
        self.game_box.load()
        if response == gtk.RESPONSE_ACCEPT:
            self.load()
        return

    # open start message dialog and preferences dialog
    def start_message(self):
        msg=gtk.Dialog("Welcome",self.window,buttons=(gtk.STOCK_OK,gtk.RESPONSE_ACCEPT))
        msg.set_size_request(350,340)
        msg.set_resizable(False)

        attrs=pango.AttrList()
        attrs.insert(pango.AttrScale(pango.SCALE_X_LARGE,0,100))
        attrs.insert(pango.AttrWeight(pango.WEIGHT_BOLD,0,100))
        welcome=gtk.Label("\n"+_("Welcome to PyXmame!"))
        welcome.set_attributes(attrs)
        
        text="\n\n"+_("The PyGTK frontend for the XMAME")+"\n"
        text+=_("Developed by Bernat Pegueroles Forcadell")+"\n"
        text+=_("Released under GNU GPLv2 license")+"\n\n"
        text+=_("Report bugs to bernat@pegueroles.cat")+"\n"
        text+=_("http://pegueroles.com/pyxmame")
        comment="<b>"+_("You have to define rompath and build the game list in Preferences dialog before begin browse games.")+"\n"
        comment+=_("Enjoy it!")+"</b>"
        
        body=gtk.Label()
        body.set_markup(text)
        body.set_line_wrap(True)
        body.set_justify(gtk.JUSTIFY_CENTER)
        
        comm=gtk.Label()
        comm.set_markup(comment)
        comm.set_line_wrap(True)
        comm.set_size_request(300,100)
        
        textbox=gtk.VBox(False,0)
        textbox.pack_start(welcome,False)
        textbox.pack_start(body,False)
        textbox.pack_start(comm,False)
        
        msg.vbox.add(textbox)
        
        pref = Preferences.Preferences(self.error,self.config)
        pref.show_all()
        
        msg.show_all()
        msg.run()
        msg.destroy()
        
        response=pref.run()
        pref.destroy()
        
        self.config.read()
        if response==gtk.RESPONSE_ACCEPT:
            self.load()
        return

    # filter games with select filter selection
    def filter_selection(self,widget=None,event=None,var=None):
        pattern=self.selects_box.get_selection()
        if pattern == None:
            self.list_box.set_focus()
            return
        match=self.gamelist.get_list(pattern)
        self.list_box.set_list(match)
        return
    
    # open about dialog
    def about(self,widget=None):
        comments=_("The PyGTK frontend for the XMAME")+"\n\n"
        comments+=_("Developed by Bernat Pegueroles Forcadell")+"\n"
        comments+=_("Report bugs to bernat@pegueroles.cat")+"\n\n"
        comments+=_("Released under GNU GPLv2 license")+"\n\n"
        comments+=_("http://pegueroles.com/pyxmame")+"\n"
        about=gtk.AboutDialog()
        about.set_name("PyXmame")
        about.set_version("0.4.0")
        about.set_comments(comments)
        about.run()
        return

    # load gamelist
    def load(self,widget=None):
        dlg=Progress.LoadDialog(self.window)
        self.gamelist.set_progress(dlg.progress)
        self.gamelist.load_list()
        self.selects_box.initialize(self.gamelist.get_selects_lists())
        dlg.destroy()
        return

    def key_press_event(self,event,key):
        key=gtk.gdk.keyval_name(key.keyval)
        if key == "Escape":
            self.exit(self, None, None)
            return True
        elif key == "Right":
            self.filter_selection()
            return True
        elif key == "Left":
            self.selects_box.set_focus()
            return True
        return
    
    # show error message
    def error(self,text):
        dlg=gtk.MessageDialog(self.window, gtk.DIALOG_MODAL,gtk.MESSAGE_ERROR,
            gtk.BUTTONS_OK,text)
        dlg.run()
        dlg.destroy()
        return
        
    def main(self):
        gtk.main()
        return

if __name__ == "__main__":
    pyx = Main()
    pyx.main()
