# GNU GPL

import string
import GameList
import Config
import gettext
import sys,os
import Progress

import pygtk
pygtk.require('2.0')
import gtk
import gobject

# The dialog to choose a image path
class ImagePathDialog(gtk.Dialog):
    def __init__(self):
        gtk.Dialog.__init__(self,title=_("Images Path"), parent=None, flags=gtk.DIALOG_MODAL, buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,
                      gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
        
        self.set_size_request(300,250)
        self.set_border_width(10)        
        self.set_resizable(False)
        self.connect("key-press-event",self.key_press)
        vbox=gtk.VBox(False,10)

        text=gtk.Label(_("You can select a path or zip file"))
        text.set_alignment(0,1)
        vbox.pack_start(text,False)

        self.path=gtk.Entry()
        self.path.set_max_length(50)
        self.path.select_region(0, len(self.path.get_text()))
        vbox.pack_start(self.path, False)

        filter=gtk.FileFilter()
        filter.add_pattern("*.zip")

        butt1 = gtk.Button(stock=gtk.STOCK_OPEN)
        butt1.connect("clicked", self.file_selection, _("Choose zip images file"),filter)
        alignment = butt1.get_children()[0]
        hbox = alignment.get_children()[0]
        image, label = hbox.get_children()
        label.set_text(_('Browse zip files'))

        butt2 = gtk.Button(stock=gtk.STOCK_OPEN)
        butt2.connect("clicked", self.file_selection, _("Choose images directory"))
        alignment = butt2.get_children()[0]
        hbox = alignment.get_children()[0]
        image, label = hbox.get_children()
        label.set_text(_('Browse directories'))

        box=gtk.VBox(False,5)
        box.set_border_width(5)        
        box.pack_start(butt1, False)
        box.pack_start(butt2, False)

        hbox=gtk.HBox()
        hbox.set_spacing(10)
        hbox.pack_start(box, True,False)


        vbox.pack_start(hbox, False)

        self.vbox.pack_start(vbox, True, True, 10)
        self.show_all()

    def key_press(self,event,key):
        if gtk.gdk.keyval_name(key.keyval) == "Return":
            self.response(gtk.RESPONSE_ACCEPT)
        if gtk.gdk.keyval_name(key.keyval) == "Escape":
            self.response(gtk.RESPONSE_CANCEL)
        return

    def get_value(self):
        return self.path.get_text()
    def set_value(self,path):
        self.path.set_text(path)
        return

    # opens file selection dialog
    def file_selection(self,widget,title,filter=None):
        if filter:
            selection=gtk.FileChooserDialog(title,self,action=gtk.FILE_CHOOSER_ACTION_OPEN,
                buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
            selection.set_filter(filter)
        else:
            selection=gtk.FileChooserDialog(title,self,action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
        response = selection.run()
        if response != gtk.RESPONSE_ACCEPT:
            selection.destroy()
            return
        self.path.set_text(selection.get_filename())
        selection.destroy()
        return

# The box that contains image paths
class Images(gtk.ScrolledWindow):
    def __init__(self,config):
        gtk.ScrolledWindow.__init__(self)
        
        cell = gtk.CellRendererText()
        self.treeview = gtk.TreeView()
        self.liststore=gtk.ListStore(gobject.TYPE_STRING)

        self.selection=self.treeview.get_selection()
        self.selection.select_path(0)
        for row in config["images"]:
            self.liststore.append([row])
        
        self.treeview.set_model(self.liststore)
        self.treeview.set_headers_visible(False)

        column=gtk.TreeViewColumn()
        self.treeview.append_column(column)
        column.pack_start(cell, True)
        column.set_attributes(cell, text=0)
        
        self.treeview.set_enable_search(False)
        self.add(self.treeview)

    # returns a list with all paths
    def get_values(self):
        values=list()
        iter=self.liststore.get_iter_first()
        while iter!=None:
            values.append(self.liststore.get_value(iter,0))
            iter=self.liststore.iter_next(iter)
        return values

    # handles the action emitted by a button
    def action(self,button,id):
        if id==gtk.STOCK_ADD:
            dlg = ImagePathDialog()
            if dlg.run() != gtk.RESPONSE_ACCEPT:
                dlg.destroy()
                return
            self.liststore.append([dlg.get_value()])
            dlg.destroy()
            return
        
        # returns because there aren't a selected path 
        (model,iter)=self.selection.get_selected()
        if iter==None:
            return
        
        if id==gtk.STOCK_EDIT:
            dlg = ImagePathDialog()
            dlg.set_value(self.liststore.get_value(iter,0))
            if dlg.run() != gtk.RESPONSE_ACCEPT:
                dlg.destroy()
                return
            self.liststore.set_value(iter,0,dlg.get_value())
            dlg.destroy()
            return
        
        if id==gtk.STOCK_REMOVE:
            self.liststore.remove(iter)
            return
        
        if id==gtk.STOCK_GO_UP:
            num=self.liststore.get_path(iter)[0]
            if num==0:
                return
            prev_iter=self.liststore.get_iter(num-1)
            self.liststore.swap(iter,prev_iter)
            return
        
        if id==gtk.STOCK_GO_DOWN:
            next_iter=self.liststore.iter_next(iter)
            if next_iter==None:
                return
            self.liststore.swap(iter,next_iter)
            return
        return

# Dialog with notebook of preferences
class Preferences(gtk.Dialog):

    def __init__(self,error,config):
        gettext.install('pyxmame','../locale',codeset='ISO-8859-15')

        gtk.Dialog.__init__(self,title=_("Preferences"), parent=None, flags=gtk.DIALOG_MODAL)

        self.connect("delete_event", self.exit)
        self.connect("key-press-event",self.key_press_event)
        self.set_border_width(10)
        self.set_resizable(False)
        self.set_size_request(600,400)
        self.set_has_separator(False)
        self.error=error
        
        self.values=dict()
        self.returned_value=gtk.RESPONSE_CANCEL

        self.config=config
        self.config.read()

        # notebook
        self.notebook=gtk.Notebook()
        self.notebook.set_tab_pos(gtk.POS_TOP)
        self.notebook.append_page(self.roms_page(),gtk.Label(_("Gamelist")))
        self.notebook.append_page(self.images_page(),gtk.Label(_("Images")))

        # the 3 down buttons of dialog
        b1 = gtk.Button(stock=gtk.STOCK_CANCEL)
        b1.connect("clicked", self.exit, False)
        b2 = gtk.Button(stock=gtk.STOCK_APPLY)
        b2.connect("clicked", self.write_config, None)
        b3 = gtk.Button(stock=gtk.STOCK_OK)
        b3.connect("clicked", self.exit, True)

        button_box=gtk.HBox(False, 10)
        button_box.pack_end(b3,False)
        button_box.pack_end(b2,False)
        button_box.pack_end(b1,False)

        box=gtk.VBox(False,10)
        box.pack_start(self.notebook)
        box.pack_start(button_box,False)

        self.vbox.add(box)
        self.show_all()

    # page of notebook that contains the catver path, the rompath path and
    # the Build Gamelist button
    def roms_page(self):
        entries_box=gtk.VBox(False,20)
        
        # rompath
        entries_box.pack_start(
                            self.entry_path_box(
                            "Choose directory where the roms are located",
                            "rompath",
                            self.config["rompath"],
                            "Select rompath",
                            None))
        # filter of ini files
        filter=gtk.FileFilter()
        filter.add_pattern("*.ini")
        # catver
        entries_box.pack_start(
                            self.entry_path_box(
                            "Choose catver.ini file",
                            "catver",
                            self.config["catver"],
                            "Select catver.ini file",
                            filter))
        
        # Build Gamelist button
        butt = gtk.Button(stock=gtk.STOCK_EXECUTE)
        butt.connect("clicked", self.build_gamelist, None)
        alignment = butt.get_children()[0]
        hbox = alignment.get_children()[0]
        image, label = hbox.get_children()
        label.set_text(_('Build Gamelist'))

        hbox=gtk.HBox(False, 10)
        hbox.pack_end(butt,False)
        button_box=gtk.VBox(False, 10)
        button_box.pack_start(gtk.HBox(),True)
        button_box.pack_end(hbox,False)
        
        text="<b>"+_("Build Gamelist")+"</b>\n\n"
        text+=_("The gamelist allows you to browse games in the PyXmame interface.")
        text+=_("You would have to define rompath and the catver.ini location before building the gamelist.")
        lab=gtk.Label()
        lab.set_markup(text)
        lab.set_alignment(0,1)
        lab.set_line_wrap(True)

        glbox=gtk.HBox()
        glbox.set_spacing(20)
        glbox.pack_start(lab,True)
        glbox.pack_start(button_box,False)

        box=gtk.VBox()
        box.set_border_width(10)
        box.set_spacing(20)
        box.pack_start(entries_box,False)
        #box.pack_start(gtk.HBox(),True)
        box.pack_start(glbox,False)

        return box 

    # image paths page of notebook
    def images_page(self):
        self.images=Images(self.config)
        self.images.set_size_request(470,-1)
        
        buttons=gtk.VBox()
        buttons.set_border_width(10)
        buttons.set_spacing(5)
        buttons.pack_start(self.image_button(gtk.STOCK_ADD),False)
        buttons.pack_start(self.image_button(gtk.STOCK_REMOVE),False)
        buttons.pack_start(self.image_button(gtk.STOCK_EDIT),False)
        buttons.pack_start(self.image_button(gtk.STOCK_GO_UP),False)
        buttons.pack_start(self.image_button(gtk.STOCK_GO_DOWN),False)
        
        box=gtk.HBox()
        box.pack_start(self.images,True)
        box.pack_start(buttons,True)
        
        text=gtk.Label()
        text.set_markup("<b>"+_("Define a list of sets of images to show the game image")+"</b>")
        text.set_alignment(0,1)
        
        vbox=gtk.VBox(False,0)
        vbox.set_border_width(10)
        vbox.set_spacing(10)
        vbox.pack_start(text,False)
        vbox.pack_start(box,True)

        return vbox

    # returns a right button of image page
    def image_button(self,id):
        butt = gtk.Button()
        image=gtk.Image()
        image.set_from_stock(id,gtk.ICON_SIZE_BUTTON)
        butt.add(image)
        butt.connect("clicked", self.images.action, id)
        return butt

    # returns a box with title, text entry and button that opens
    # file selection
    def entry_path_box(self,text,key,default_value,title,filter):
        textview=gtk.Label()
        textview.set_markup("<b>"+_(text)+"</b>")
        textview.set_alignment(0,1)
    
        self.values[key]=gtk.Entry()
        self.values[key].set_max_length(200)
        self.values[key].select_region(0, len(self.values[key].get_text()))
        self.values[key].set_text(default_value)

        button=gtk.Button(stock=gtk.STOCK_OPEN)
        button.connect("clicked",self.file_selection,key,title,filter)

        values_box=gtk.HBox(False,10)
        values_box.pack_start(self.values[key])
        values_box.pack_start(button,False)

        entry_box=gtk.VBox(False,0)
        entry_box.pack_start(textview)
        entry_box.pack_start(values_box)
        
        return entry_box

    # opens file selection dialog
    def file_selection(self,widget,key,title,filter):
        if filter:
            selection=gtk.FileChooserDialog(title,self,action=gtk.FILE_CHOOSER_ACTION_OPEN,
                buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
            selection.set_filter(filter)
        else:
            selection=gtk.FileChooserDialog(title,self,action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
        response = selection.run()
        if response != gtk.RESPONSE_ACCEPT:
            selection.destroy()
            return
        self.values[key].set_text(selection.get_filename())
        selection.destroy()
        return

    # writes the defined configuration 
    def write_config(self,widget=None,event=None):
        self.config["rompath"]=self.values["rompath"].get_text()
        self.config["catver"]=self.values["catver"].get_text()
        self.config["images"]=self.images.get_values()
        self.config.write()
        self.config.read()
        return

    # build the gamelist
    def build_gamelist(self,widget=None,event=None):
        self.config["rompath"]=self.values["rompath"].get_text()
        self.config["catver"]=self.values["catver"].get_text()
        build=Progress.BuildDialog(self)
        self.gamelist=GameList.GameList(self.config)
        self.gamelist.set_progress(build.progress)
        
        self.gamelist.build_list()
        build.destroy()
        self.returned_value=gtk.RESPONSE_ACCEPT
        self.config.write()
        self.config.read()
        return

    def exit(self, widget=None, write=False, data=None):
        if write:
            self.write_config()
        self.response(self.returned_value)
        return True

    def key_press_event(self,event,key):
        if gtk.gdk.keyval_name(key.keyval) == "Escape":
            self.exit()
        return
