# GNU GPL

import pygtk
pygtk.require('2.0')
import gtk
import gobject

# This box contains the select filter
class Select(gtk.ScrolledWindow):
    def __init__(self,filter_selection):
        gtk.ScrolledWindow.__init__(self)
        
        self.treeview = gtk.TreeView()
        self.liststore=gtk.ListStore(gobject.TYPE_STRING)
        self.selection=self.treeview.get_selection()
        self.selection.set_mode(gtk.SELECTION_MULTIPLE)
        self.selection.select_path(0)
        self.set_size_request(200,-1)

        self.treeview.set_model(self.liststore)
        self.treeview.set_headers_visible(False)

        column=gtk.TreeViewColumn()
        self.treeview.append_column(column)
        cell = gtk.CellRendererText()
        column.pack_start(cell, True)
        column.set_attributes(cell, text=0)

        self.treeview.connect("row-activated",filter_selection)

        self.treeview.set_enable_search(False)
        self.treeview.set_cursor(0,column)
        self.add(self.treeview)

        return
        
    # returns the list selected 
    def get_selection(self):
        selection=list()
        (model,paths)=self.selection.get_selected_rows()
        for selec in paths:
            iter=model.get_iter(selec[0])
            selection.append(model.get_value(iter,0))
        return selection

    # sets the selection list
    def set_list(self,list):
        self.liststore.clear()
        for row in list:
            self.liststore.append([row])
        return

# This box contains contains the select filter and the drop down menu that
# changes the content of select filter
class Selects(gtk.VBox):
    def __init__(self,config,filter_selection):
        gtk.VBox.__init__(self)

        self.config = config
        self.initialize()

        # select filter box
        self.select_box= Select(filter_selection)
        self.lists=dict()
        for select in config["selects"]:
            self.lists[select]=list()

        # drop down menu (combobox)
        liststore = gtk.ListStore(gobject.TYPE_STRING)
        self.combobox = gtk.ComboBox(liststore)
        cell = gtk.CellRendererText()
        self.combobox.pack_start(cell, True)
        self.combobox.add_attribute(cell, 'text', 0)
        for select in config["selects"]:
            liststore.append([_(select.capitalize())])
        self.combobox.connect("changed",self.change_select)
        self.combobox.set_active(config["selects"].index(config["default_select"]))

        self.pack_start(self.combobox, False, True, 0)
        self.pack_start(self.select_box, True, True, 0)
        return

    # this function is called when combobox value changes
    def change_select(self, item=None):
        self.current_select_box=self.combobox.get_active_text().lower()
        self.select_box.set_list(self.lists[self.current_select_box])
        col=self.select_box.treeview.get_column(0)
        self.select_box.treeview.set_cursor(0,col)
        self.select_box.treeview.grab_focus()
        self.initialize()
        return

    # returns a dictionary with select filter values
    #    pattern[combobox_value]= regular expression of selected values
    def get_selection(self):
        selection=self.select_box.get_selection()
        if self.lastselection == selection:
            return None
        self.lastselection=selection
        pattern=dict()
        if _('Any') not in selection:
            pattern[self.current_select_box]=""
            for select in selection:
                pattern[self.current_select_box]=pattern[self.current_select_box]+"^"+select+"$"+'|'
            pattern[self.current_select_box]=pattern[self.current_select_box][:-1].replace('*','\*')
        return pattern  

    def initialize(self,lists=None):
        self.lastselection = None
        if lists:
            self.lists=lists
            self.change_select()
        return
        
    def set_focus(self):
        self.select_box.treeview.grab_focus()
        return
